function [C, matches] = regularize(M, N, C_init, matches_init, n_sample_matches, params)

fprintf('Sampling %d farthest matches... ', n_sample_matches);

fps = fps_euclidean(M.VERT, n_sample_matches, 1);
FG = compute_indicator_functions({M,N}, [fps matches_init(fps)]', 0.01*sqrt(sum(calc_tri_areas(M))));
F = FG{1};
G = FG{2};

fprintf('done.\n');

k = params.n_eigen;
W = zeros(k);
for i=1:k
    for j=1:k
        slope = 1;
        direction = [1 slope];
        direction = direction./norm(direction);
        W(i,j) = exp(-0.03*sqrt(i.^2 + j.^2))*norm(cross([direction 0], [i,j, 0]-[1 1 0]));        
    end
end
%imagesc(W);
d=ones(1,k);
D = repmat(d,k,1);

mu1 = 1;   % slanted-diagonal mask
mu2 = 1e3; % sub-orthogonality

A = M.evecs'*M.S*F;
B = N.evecs'*N.S*G;

manifold = euclideanfactory(k,k);
problem = {};

problem.M = manifold;

problem.cost = @(C) (...
    sum(sum((C*A-B).^2).^0.5) + ...
    mu1 * norm(C.*W,'fro')^2 + ...
    mu2 * (norm(C'*C,'fro')^2 - sum(diag(C'*C).^2) + sum((diag(C'*C) - d').^2) ));

problem.egrad = @(C) (...
    norm_21_gradient(C,A,B) + ...
    mu1 * 2 * C.*W.*W + ...
    mu2 * 4*(C*C'*C - C.*D ));

%figure, checkgradient(problem)

options.maxiter = 3e3;
options.tolgradnorm = 1e-06;
options.minstepsize = 1e-06;
options.verbosity = 0;

fprintf('Regularizing... ');

[C, final_cost, ~, ~] = conjugategradient(problem, C_init, options);

fprintf('done.\nFinal cost: %f\n', final_cost);

[C, matches] = run_icp(N, M, params.n_eigen, C, 20);

end
